/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#if !defined (HOTSPOT_H)
#define HOTSPOT_H

#include <string>
#include <windows.h>
#include <lsapi/lsapi.h>

#include "CustomTypes.h"


class Hotspot
{
public:

	Hotspot(int const &id, HWND hwnd);

	virtual ~Hotspot();

public:

	MouseInOut const Check();

	virtual void Move(Point const &pos) = 0;

	virtual void MoveBy(Point const &dpos) = 0;
	
	virtual void Resize(Point const &dim) = 0;

	virtual void ResizeBy(Point const &ddim) = 0;

	virtual void Reposition(Point const &pos, Point const &dim) = 0;

	virtual void RepositionBy(Point const &dpos, Point const &ddim) = 0;

	bool const IsActive() const;

	virtual void SetActive(bool const state);

	virtual void Toggle();

	// setting interval to 0 hinders the spot from actively working
	void SetInterval(int const &int_ms);

	void SetActionEnter(std::string const &action);

	void SetActionLeave(std::string const &action);

	void ExecuteAction(std::string const &action);

	int const GetId() const;

	bool const IsInside();

	virtual bool const IsInside(Point const& mouse_pos) = 0;

protected:

	void StartTimer();

	void StopTimer();

private:

	void GetMousePosition(Point &point);

protected:

	int x_;

	int y_;

	int id_;

	bool active_;

private:

	std::string name_;

	std::string action_enter_;

	std::string action_leave_;

	bool last_inside_;

	int interval_;

	bool has_group_;

	HWND hwnd_;

};

#endif
