/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "xHotspot.h"

#include <string>

#include "stdafx.h"
#include "Util.h"
#include "HotspotGroup.h"

// the instance of our main class
extern xHotspot *xhotspot;

int monitor_width = 0;
int monitor_height = 0;


xHotspot::xHotspot()
: spotmanager_(hInst)
{}

xHotspot::~xHotspot()
{}

void xHotspot::LoadHotspots()
{
	// first load real group spots, then normal spots
	StringList grouplist = util::GetNameList("*xhotspotgroup");
	for(StringList::iterator name = grouplist.begin(); name != grouplist.end(); ++name)
	{
		CreateSpot(*name, true, grouplist);
	}

	StringList namelist = util::GetNameList("*xhotspot");
	for(StringList::iterator name = namelist.begin(); name != namelist.end(); ++name)
	{
		//LSLogPrintf(4, "xHotspot", "loadhotspots name=%s",(*name).c_str());
		CreateSpot(*name, false, grouplist);
	}

	spotmanager_.InitSpots();
}

Hotspot *xHotspot::CreateSpot(std::string const &name, bool const is_group, StringList const &real_groups)
{
	HotspotSettings set;

	ReadSpotSettings(name, set);

	bool is_real_group = false;

	if (is_group)
	{
		is_real_group = util::Find(name, real_groups);
		if (is_real_group)
			set.type = RealGroup;
		else
			set.type = NormalGroup;
	}

	Hotspot *spot = spotmanager_.AddSpot(name, set.type, hInst);
	spot->Reposition(set.position, set.size);
	spot->SetInterval(set.update_interval);
	spot->SetActionEnter(set.on_enter);
	spot->SetActionLeave(set.on_leave);

	//LSLogPrintf(4, "xHotspot", "createspot name=%s type=%d",name.c_str(), set.type);

	if (set.group != "")
	{
		HotspotGroup *groupspot = static_cast<HotspotGroup * >(spotmanager_.GetSpot(set.group));
		if (groupspot == 0)
			groupspot = static_cast<HotspotGroup * >(CreateSpot(set.group, true, real_groups));
		
		groupspot->AddSubspot(spot);

		if (util::Find(set.group, real_groups))
		// the spot's group is a real group
		{
			//LSLogPrintf(4, "xHotspot", "createspot put %s in REAL group %s",name.c_str(), set.group.c_str());

			// stop this spot from checking by itself. it can still be "active", but
			// the timer won't be started. checks are done by the group instead
			spot->SetInterval(0);
		}
		else
		// the spot's group is a normal group
		{
			//LSLogPrintf(4, "xHotspot", "createspot put %s in group %s",name.c_str(), set.group.c_str());
			/*HotspotGroup *groupspot = static_cast<HotspotGroup * >(spotmanager_.GetSpot(set.group));
			if (groupspot == 0)
				groupspot = static_cast<HotspotGroup * >(CreateSpot(set.group, true, real_groups));
		
			groupspot->AddSubspot(spot);*/
		}

	}

	if(!is_group || is_real_group) // don't init normal groups
		spotmanager_.AddInitSpot(spot, set.enabled);

	return spot;
}


void xHotspot::ReadSpotSettings(std::string const &name, HotspotSettings &set)
{
/*
	settings to read:
	*xhotspot h
	hx 0
	hy 0
	hwidth 1
	hheight 1
	htype rectangular|elliptic
	hupdateinterval 100
	henabled true
	haddtogroup group
	honenter !bang
	honleave !bang
*/
	set.position.x = util::parse_string_to_int(util::GetSetting(name, "X", "0", false), monitor_width);
	set.position.y = util::parse_string_to_int(util::GetSetting(name, "Y", "0", false), monitor_height);
	set.size.x = util::parse_string_to_int(util::GetSetting(name, "Width", "1", false), monitor_width);
	set.size.y = util::parse_string_to_int(util::GetSetting(name, "Height", "1", false), monitor_height);
	set.type = util::string_to_type(util::GetSetting(name, "Type", "Rectangular", false));
	set.update_interval = util::string_to_int(util::GetSetting(name, "UpdateInterval", "100", false));
	set.enabled = util::string_to_bool(util::GetSetting(name, "Enabled", "true", false));
	set.group = util::GetSetting(name, "AddToGroup", "", false);
	set.on_enter = util::GetSetting(name, "OnEnter", "!none", true);
	set.on_leave = util::GetSetting(name, "OnLeave", "!none", true);
}

void xHotspot::BangSpotCreate(std::string const &name)
{
	// only create if it hasn't been created yet
	if(!spotmanager_.GetSpot(name))
	{
		StringList group_list = util::GetNameList("*xHotspotGroup");
		if(util::Find(name, group_list))
			CreateSpot(name, true, group_list);
		else
			CreateSpot(name, false, group_list);
	}
}

void xHotspot::BangSpotDestroy(std::string const &name)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		std::string group = util::GetSetting(name, "AddToGroup", "", false);
		if(!group.empty())
		// remove from group
		{
			/*if(util::Find(group, util::GetNameList("*xHotspotGroup")))
				static_cast<RealGroupHotspot * >(spotmanager_.GetSpot(group))->RemoveSubspot(spot);
			else
				static_cast<GroupHotspot * >(spotmanager_.GetSpot(group))->RemoveSubspot(spot);*/
			static_cast<HotspotGroup * >(spotmanager_.GetSpot(group))->RemoveSubspot(spot);
		}
		spotmanager_.RemoveSpot(name);
	}
}

void xHotspot::BangSpotSetType(std::string const &name, HotspotType const &type)
{
	spotmanager_.SetType(name, type);
}

void xHotspot::BangSpotEnable(std::string const &name)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
		spot->SetActive(true);
}

void xHotspot::BangSpotDisable(std::string const &name)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
		spot->SetActive(false);
}

void xHotspot::BangSpotToggle(std::string const &name)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		spot->Toggle();
	}
}

void xHotspot::BangSpotMove(std::string const &name, int const &x, int const &y)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		Point p;
		p.x = x;
		p.y = y;
		spot->Move(p);
	}
}

void xHotspot::BangSpotMoveBy(std::string const &name, int const &dx, int const &dy)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		Point p;
		p.x	= dx;
		p.y = dy;
		spot->MoveBy(p);
	}
}

void xHotspot::BangSpotResize(std::string const &name, int const &width, int const &height)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		Point p;
		p.x = width;
		p.y = height;
		spot->Resize(p);
	}
}

void xHotspot::BangSpotResizeBy(std::string const &name, int const &dwidth, int const &dheight)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		Point p;
		p.x = dwidth;
		p.y = dheight;
		spot->ResizeBy(p);
	}
}

void xHotspot::BangSpotReposition(std::string const &name, int const &x, int const &y, int const &width, int const &height)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		Point p, s;
		p.x = x;
		p.y = y;
		s.x = width;
		s.y = height;
		spot->Reposition(p, s);
	}
}

void xHotspot::BangSpotRepositionBy(std::string const &name, int const &dx, int const &dy, int const &dwidth, int const &dheight)
{
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		Point p, s;
		p.x = dx;
		p.y = dy;
		s.x = dwidth;
		s.y = dheight;
		spot->RepositionBy(p, s);
	}
}

void xHotspot::BangSpotIfInside(std::string const &name, std::string const &cmd_true, std::string const &cmd_false)
{
	//LSLogPrintf(4, "xHotspot", "xhotspot::bangspotifinside");
	Hotspot *spot = spotmanager_.GetSpot(name);
	if(spot)
	{
		if(spot->IsInside())
		{
			//LSLogPrintf(4, "xHotspot", "xhotspot::bangspotifinside inside true");
			spot->ExecuteAction(cmd_true);
		}
		else
		{
			//LSLogPrintf(4, "xHotspot", "xhotspot::bangspotifinside inside false!");
			spot->ExecuteAction(cmd_false);
		}
		//LSLogPrintf(4, "xHotspot", "xhotspot::bangspotifinside after 1");
	}
	//LSLogPrintf(4, "xHotspot", "xhotspot::bangspotifinside after 2");
}


/* Bang Commands */

void BangxHotspotCreate(HWND caller, LPCSTR args)
{
	char name[MAX_CHAR];
	char *buffer[] = {name};

	LCTokenize(args, buffer, 1, 0);

	xhotspot->BangSpotCreate(name);
}

void BangxHotspotDestroy(HWND caller, LPCSTR args)
{
	char name[MAX_CHAR];
	char *buffer[] = {name};
	
	LCTokenize(args, buffer, 1, 0);

	xhotspot->BangSpotDestroy(name);
}

void BangxHotspotSetType(HWND caller, LPCSTR args)
{
	char name[MAX_CHAR];
	char ctype[16];
	char *buffer[] = {name, ctype};
	
	LCTokenize(args, buffer, 2, 0);

	HotspotType type;
	if (ctype == "elliptic")
		type = Elliptic;
	else
		type = Rectangular;
    xhotspot->BangSpotSetType(name, type);
}

void BangxHotspotEnable(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char *buffer[] = {cname};
	
	LCTokenize(args, buffer, 1, 0);

	std::string name = cname;
	xhotspot->BangSpotEnable(name);
}

void BangxHotspotDisable(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char *buffer[] = {cname};
	
	LCTokenize(args, buffer, 1, 0);

	std::string name = cname;
	xhotspot->BangSpotDisable(name);
}

void BangxHotspotToggle(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char *buffer[] = {cname};
	
	LCTokenize(args, buffer, 1, 0);
	std::string name = cname;
	xhotspot->BangSpotToggle(name);
}

void BangxHotspotMove(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char cx[16], cy[16];
	char *buffer[] = {cname, cx, cy};
	
	LCTokenize(args, buffer, 3, 0);

	std::string name = cname;
	int x = util::parse_char_to_int(cx, monitor_width);
	int y = util::parse_char_to_int(cy, monitor_height);
	xhotspot->BangSpotMove(name, x, y);
}

void BangxHotspotMoveBy(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char cdx[16], cdy[16];
	char *buffer[] = {cname, cdx, cdy};
	
	LCTokenize(args, buffer, 3, 0);

	std::string name = cname;
	int dx = util::parse_char_to_int(cdx, monitor_width);
	int dy = util::parse_char_to_int(cdy, monitor_height);
	xhotspot->BangSpotMoveBy(name, dx, dy);
}

void BangxHotspotResize(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char cwidth[16], cheight[16];
	char *buffer[] = {cname, cwidth, cheight};
	
	LCTokenize(args, buffer, 3, 0);

	std::string name = cname;
	int width = util::parse_char_to_int(cwidth, monitor_width);
	int height = util::parse_char_to_int(cheight, monitor_height);
	xhotspot->BangSpotResize(name, width, height);
}

void BangxHotspotResizeBy(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char cdwidth[16], cdheight[16];
	char *buffer[] = {cname, cdwidth, cdheight};
	
	LCTokenize(args, buffer, 3, 0);

	std::string name = cname;
	int dwidth = util::parse_char_to_int(cdwidth, monitor_width);
	int dheight = util::parse_char_to_int(cdheight, monitor_height);
	xhotspot->BangSpotResizeBy(name, dwidth, dheight);
}

void BangxHotspotReposition(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char cx[16], cy[16];
	char cwidth[16], cheight[16];
	char *buffer[] = {cname, cx, cy, cwidth, cheight};
	
	LCTokenize(args, buffer, 5, 0);

	std::string name = cname;
	int x = util::parse_char_to_int(cx, monitor_width);
	int y = util::parse_char_to_int(cy, monitor_height);
	int width = util::parse_char_to_int(cwidth, monitor_width);
	int height = util::parse_char_to_int(cheight, monitor_height);
	xhotspot->BangSpotReposition(name, x, y, width, height);
}

void BangxHotspotRepositionBy(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char cdx[16], cdy[16];
	char cdwidth[16], cdheight[16];
	char *buffer[] = {cname, cdx, cdy, cdwidth, cdheight};
	
	LCTokenize(args, buffer, 5, 0);

	std::string name = cname;
	int dx = util::parse_char_to_int(cdx, monitor_width);
	int dy = util::parse_char_to_int(cdy, monitor_height);
	int dwidth = util::parse_char_to_int(cdwidth, monitor_width);
	int dheight = util::parse_char_to_int(cdheight, monitor_height);
	xhotspot->BangSpotRepositionBy(name, dx, dy, dwidth, dheight);
}

void BangxHotspotIfInside(HWND caller, LPCSTR args)
{
	char cname[MAX_CHAR];
	char ccmd_true[MAX_CHAR];
	char ccmd_false[MAX_CHAR];
	char *buffer[] = {cname, ccmd_true, ccmd_false};

	LCTokenize(args, buffer, 3, 0);
	
	//LSLogPrintf(4, "xHotspot", "bangifinside name=%s cmd_true=%s cmd_false=%s", cname, ccmd_true, ccmd_false);
	
	std::string name = cname;
	std::string cmd_true = ccmd_true;
	std::string cmd_false = ccmd_false;
	xhotspot->BangSpotIfInside(name, cmd_true, cmd_false);
}
