/******************************************************************************

This is a part of the xHotspot Source code.

Copyright (C) 2007 Sascha Liebsch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#if !defined (XHOTSPOT_H)
#define XHOTSPOT_H

#include "HotspotManager.h"
#include "CustomTypes.h"

static HWND hwnd;
static HINSTANCE hInst;


LRESULT CALLBACK MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

// Bangs
void BangxHotspotCreate(HWND caller, LPCSTR args);
void BangxHotspotDestroy(HWND caller, LPCSTR args);
void BangxHotspotSetType(HWND caller, LPCSTR args);
void BangxHotspotEnable(HWND caller, LPCSTR args);
void BangxHotspotDisable(HWND caller, LPCSTR args);
void BangxHotspotToggle(HWND caller, LPCSTR args);
void BangxHotspotMove(HWND caller, LPCSTR args);
void BangxHotspotMoveBy(HWND caller, LPCSTR args);
void BangxHotspotResize(HWND caller, LPCSTR args);
void BangxHotspotResizeBy(HWND caller, LPCSTR args);
void BangxHotspotReposition(HWND caller, LPCSTR args);
void BangxHotspotRepositionBy(HWND caller, LPCSTR args);
void BangxHotspotIfInside(HWND caller, LPCSTR args);


extern "C"
{
  __declspec( dllexport ) int initModuleEx(HWND hParent, HINSTANCE hInstance, LPCTSTR szPath);
  __declspec( dllexport ) void quitModule(HINSTANCE hInstance);
}


class xHotspot {

public:

	xHotspot();

	~xHotspot();

public:

	// Loads Hotspots and starts initialisation afterwards.
	void LoadHotspots();

	void BangSpotCreate(std::string const &name);

	void BangSpotDestroy(std::string const &name);

	void BangSpotSetType(std::string const &name, HotspotType const &type);

	void BangSpotEnable(std::string const &name);

	void BangSpotDisable(std::string const &name);

	void BangSpotToggle(std::string const &name);

	void BangSpotMove(std::string const &name,
		int const &x, int const &y);

	void BangSpotMoveBy(std::string const &name,
		int const &dx, int const &dy);

	void BangSpotResize(std::string const &name,
		int const &width, int const &height);

	void BangSpotResizeBy(std::string const &name,
		int const &dwidth, int const &dheight);

	void BangSpotReposition(std::string const &name,
		int const &x, int const &y, int const &width, int const &height);

	void BangSpotRepositionBy(std::string const &name, 
		int const &dx, int const &dy, int const &dwidth, int const &dheight);

	void BangSpotIfInside(std::string const &name,
		std::string const &cmd_true, std::string const &cmd_false);

private:

	/*
	 *  Creates a Hotspot with the given name and puts it into its group,
	 *  if neccessary.
	 */
	Hotspot *CreateSpot(std::string const &name, bool const is_group, StringList const &real_groups);

	/*
	 *  Reads the settings for the Hotspot with the given name and puts
	 *  them into the given settings var.
	 */
	void ReadSpotSettings(std::string const &name, HotspotSettings &set);

private:

	HotspotManager spotmanager_;

};

#endif